# this file is generated by Lino
from django.utils.translation import ugettext
ugettext('Next page')
ugettext('Please wait...')
ugettext('Action not available on phantom record.')
ugettext('Next')
ugettext('Go to record')
ugettext('Reload current record')
ugettext('Save changes to current record ?')
ugettext('Edit text in own window')
ugettext("Field '{0}' is disabled")
ugettext('Confirmation')
ugettext('No data to display.')
ugettext('Alert')
ugettext('Previous page')
ugettext('Cannot show detail for empty foreign key.')
ugettext('This record is disabled')
ugettext('View')
ugettext('Displaying {0} - {1} of {2}')
ugettext('Team view')
ugettext('Last')
ugettext('Edit help texts for fields on this model.')
ugettext('Last page')
ugettext('of {0}')
ugettext('Help Text Editor')
ugettext('Save Grid Configuration')
ugettext('Page')
ugettext('First')
ugettext('Show this panel in own window')
ugettext('[parameters]')
ugettext('This field is disabled')
ugettext('Layout Editor')
ugettext('Save changes to text ?')
ugettext('No more records to display. Detail window has been closed.')
ugettext('First page')
ugettext('Select another view of this report')
ugettext('Cannot show MTI child if checkbox is off.')
ugettext('Previous')
ugettext('Edit Detail layout')
ugettext('Enter a date in format YYYY-MM-DD (use zeroes for unknown parts).')
